@@ -3,16 +3,20 @@ use crate::resources::CellMap;
 use bevy::ecs::component::Component;
 use bevy::log;
 use bevy::prelude::*;
+use bevy::tasks::ComputeTaskPool;
+use std::sync::{Arc, RwLock};
 
-pub fn handle_cells<C, S>(
+pub fn handle_cells<C, S, const BATCH_SIZE: usize>(
     mut commands: Commands,
+    pool: Res<ComputeTaskPool>,
     query: Query<(Entity, &C, &S)>,
     map: Res<CellMap<C>>,
 ) where
     C: Cell + Component,
     S: CellState + Component,
 {
-    for (entity, cell, state) in query.iter() {
+    let vec = Arc::new(RwLock::new(Vec::new()));
+    query.par_for_each(&pool, BATCH_SIZE, |(entity, cell, state)| {
         let neighbor_coords = cell.neighbor_coordinates();
         let neighbor_cells = map.get_cell_entities(&neighbor_coords);
         let neighbor_states: Vec<S> = neighbor_cells
@@ -27,9 +31,13 @@ pub fn handle_cells<C, S>(
             .collect();
         let new_state = state.new_cell_state(&neighbor_states);
         if &new_state != state {
-            let mut entity_cmd = commands.entity(entity);
-            entity_cmd.insert(new_state);
+            let mut lock = vec.write().unwrap();
+            lock.push((entity, new_state));
         }
+    });
+    let lock = vec.read().unwrap();
+    for (e, s) in lock.iter() {
+        commands.entity(*e).insert(s.clone());
     }
 }
 
